@@ -442,21 +442,28 @@ Rcpp::loadModule("istaMixedPenaltySEM_cpp", TRUE)
 Rcpp::loadModule("istaMixedPenaltymgSEM_cpp", TRUE)
 Rcpp::loadModule("glmnetEnetSEM_cpp", TRUE)
 Rcpp::loadModule("glmnetEnetMgSEM_cpp", TRUE)
+Rcpp::loadModule("glmnetScadSEM_cpp", TRUE)
+Rcpp::loadModule("glmnetScadMgSEM_cpp", TRUE)
+Rcpp::loadModule("glmnetCappedL1SEM_cpp", TRUE)
+Rcpp::loadModule("glmnetCappedL1MgSEM_cpp", TRUE)
+Rcpp::loadModule("glmnetMcpSEM_cpp", TRUE)
+Rcpp::loadModule("glmnetMcpMgSEM_cpp", TRUE)
+Rcpp::loadModule("glmnetLspSEM_cpp", TRUE)
+Rcpp::loadModule("glmnetLspMgSEM_cpp", TRUE)
+Rcpp::loadModule("glmnetMixedSEM_cpp", TRUE)
+Rcpp::loadModule("glmnetMixedMgSEM_cpp", TRUE)
+
 Rcpp::loadModule("bfgsEnetSEM_cpp", TRUE)
 Rcpp::loadModule("bfgsEnetMgSEM_cpp", TRUE)
 
 # General Purpose
 Rcpp::loadModule("glmnetEnetGeneralPurpose_cpp", TRUE)
+Rcpp::loadModule("glmnetMixedPenaltyGeneralPurpose_cpp", TRUE)
 Rcpp::loadModule("istaEnetGeneralPurpose_cpp", TRUE)
-Rcpp::loadModule("istaLspGeneralPurpose_cpp", TRUE)
-Rcpp::loadModule("istaMcpGeneralPurpose_cpp", TRUE)
-Rcpp::loadModule("istaScadGeneralPurpose_cpp", TRUE)
-Rcpp::loadModule("istaCappedL1GeneralPurpose_cpp", TRUE)
+Rcpp::loadModule("istaMixedPenaltyGeneralPurpose_cpp", TRUE)
 
 # general Purpose with Cpp
 Rcpp::loadModule("glmnetEnetGeneralPurposeCpp_cpp", TRUE)
+Rcpp::loadModule("glmnetMixedPenaltyGeneralPurposeCpp_cpp", TRUE)
 Rcpp::loadModule("istaEnetGeneralPurposeCpp_cpp", TRUE)
-Rcpp::loadModule("istaLspGeneralPurposeCpp_cpp", TRUE)
-Rcpp::loadModule("istaMcpGeneralPurposeCpp_cpp", TRUE)
-Rcpp::loadModule("istaScadGeneralPurposeCpp_cpp", TRUE)
-Rcpp::loadModule("istaCappedL1GeneralPurposeCpp_cpp", TRUE)
\ No newline at end of file
+Rcpp::loadModule("istaMixedPenaltyGeneralPurposeCpp_cpp", TRUE)
\ No newline at end of file
